==> pymobiledevice3-2.21.0-py3-none-any.whl: pymobiledevice3/remote/utils.py <==
import contextlib
import platform
from typing import Generator, List

import psutil
import requests

from pymobiledevice3.exceptions import AccessDeniedError, TunneldConnectionError
from pymobiledevice3.remote.remote_service_discovery import RemoteServiceDiscoveryService

REMOTED_PATH = '/usr/libexec/remoted'

TUNNELD_DEFAULT_ADDRESS = ('127.0.0.1', 5555)


def get_tunneld_devices(tunneld_address=TUNNELD_DEFAULT_ADDRESS) -> List[RemoteServiceDiscoveryService]:
    try:
        # Get the list of tunnels from the specified address
        resp = requests.get(f'http://{tunneld_address[0]}:{tunneld_address[1]}')
        tunnels = resp.json()
    except requests.exceptions.ConnectionError:
        raise TunneldConnectionError()

    rsds = [RemoteServiceDiscoveryService(tunnel_address) for tunnel_udid, tunnel_address in tunnels.items()]
    for rsd in rsds:
        rsd.connect()
    return rsds


def get_remoted_process() -> psutil.Process:
    for process in psutil.process_iter():
        if process.pid == 0:
            # skip kernel task
            continue
        try:
            if process.exe() == REMOTED_PATH:
                return process
        except (psutil.ZombieProcess, psutil.NoSuchProcess):
            continue


def stop_remoted_if_required() -> None:
    if platform.system() != 'Darwin':
        # only Darwin systems require it
        return

    remoted = get_remoted_process()
    if remoted is None:
        return
    if remoted.status() == 'stopped':
        # process already stopped, we don't need to do anything
        return

    try:
        remoted.suspend()
    except psutil.AccessDenied:
        raise AccessDeniedError()


def resume_remoted_if_required() -> None:
    if platform.system() != 'Darwin':
        # only Darwin systems require it
        return

    remoted = get_remoted_process()
    if remoted is None:
        return
    if remoted.status() == 'running':
        # process already running, we don't need to do anything
        return

    try:
        remoted.resume()
    except psutil.AccessDenied:
        raise AccessDeniedError()


@contextlib.contextmanager
def stop_remoted() -> Generator[None, None, None]:
    stop_remoted_if_required()
    try:
        yield
    finally:
        resume_remoted_if_required()

==> pymobiledevice3-3.0.0-py3-none-any.whl: pymobiledevice3/remote/utils.py <==
import contextlib
import platform
import sys
from typing import Generator, List

import psutil

from pymobiledevice3.bonjour import DEFAULT_BONJOUR_TIMEOUT, browse_remoted
from pymobiledevice3.exceptions import AccessDeniedError
from pymobiledevice3.remote.remote_service_discovery import RSD_PORT, RemoteServiceDiscoveryService

REMOTED_PATH = '/usr/libexec/remoted'


async def get_rsds(bonjour_timeout: float = DEFAULT_BONJOUR_TIMEOUT) -> List[RemoteServiceDiscoveryService]:
    result = []
    with stop_remoted():
        for answer in await browse_remoted(timeout=bonjour_timeout):
            for ip in answer.ips:
                rsd = RemoteServiceDiscoveryService((ip, RSD_PORT))
                try:
                    rsd.connect()
                except ConnectionRefusedError:
                    continue
                except OSError:
                    continue
                result.append(rsd)
    return result


def get_remoted_process() -> psutil.Process:
    for process in psutil.process_iter():
        if process.pid == 0:
            # skip kernel task
            continue
        try:
            if process.exe() == REMOTED_PATH:
                return process
        except (psutil.ZombieProcess, psutil.NoSuchProcess):
            continue


def stop_remoted_if_required() -> None:
    if platform.system() != 'Darwin':
        # only Darwin systems require it
        return

    remoted = get_remoted_process()
    if remoted is None:
        return
    if remoted.status() == 'stopped':
        # process already stopped, we don't need to do anything
        return

    try:
        remoted.suspend()
    except psutil.AccessDenied:
        raise AccessDeniedError()


def resume_remoted_if_required() -> None:
    if platform.system() != 'Darwin':
        # only Darwin systems require it
        return

    remoted = get_remoted_process()
    if remoted is None:
        return
    if remoted.status() == 'running':
        # process already running, we don't need to do anything
        return

    try:
        remoted.resume()
    except psutil.AccessDenied:
        raise AccessDeniedError()


@contextlib.contextmanager
def stop_remoted() -> Generator[None, None, None]:
    stop_remoted_if_required()
    try:
        yield
    finally:
        resume_remoted_if_required()


def install_driver_if_required() -> None:
    if sys.platform == 'win32':
        import pywintunx_pmd3
        pywintunx_pmd3.install_wetest_driver()
